/*
 * AnalogOut.c
 *
 * This program takes a base frequency, a base amplitude, the harmonic 
 * number, the harmonic amplitude, the phase offset of the harmonic, the 
 * maximum number of samples, and an interrupt service routine frequency,
 * and calculates and outputs a fourier series made of the two input components.
 *
 * BASEFREQ:  The frequency of the low frequency component (Hz).
 *            BASEFREQ < 500 (The rise time of the lowpass-filtered wave
 *            is not fast enough to accurately represent frequencies higher
 *            than about 500 Hz).
 * BASEAMP:   The amplitude of the low frequency component (volts).
 *            BASEAMP <= 1 (Otherwise there might be clipping at the top and
 *            bottom of the waveform).
 * HARMONIC:  The other sinusoid is at HARMONIC*BASEFREQ Hz; must be
 *            an integer value > 1.
 *            HARMONIC < 500/BASEFREQ (So that the high frequency component
 *            doesn't get above 500 Hz either).
 * HARMAMP:   The amplitude of the other sinusoid (volts).
 *            HARMAMP <= BASEAMP (Harmonic amplitudes should be smaller than
 *            the base amplitude).
 * PHASE:     The phase of the second sinusoid relative to
 *            base sinusoid (degrees).
 *            PHASE < 360 (Because of the 360 degree periodicity of phase).
 *
 * For the lowpass filter, I used R = 10000 ohms and C = 0.01 uF.  This creates
 * a cutoff frequency of 1600 Hz and a rise time (to 63%) of RC = 0.1 ms.
 * This rise time is fast enough to accomodate inputs below the bounds above.
 *
 * HISTORY:
 * 02/14/2011  Broke a few long lines into several lines and converted ternary 
 *				operator (see Essential C, p. 11) into if/else statement. -Nelson
 * 02/05/2011  Created by a student in ME333
 *
 */


#include <plib.h>
					
#define SYS_FREQ  	80000000		// clock frequency set by bootloader

// Define the prescaler and math constants
#define PRE_SCALER	8
#define PI			3.1415927

// Define inputs to program
#define BASEFREQ	20
#define BASEAMP		1
#define HARMONIC	3
#define HARMAMP		.25
#define PHASE		145

// Define parameters of the program
#define MAXSAMPS	1000
#define ISRFREQ		10000

// initialize useful global variables
int count = 0;
int dutyvec[MAXSAMPS];
int numsamps;

int main(void) {
  int ii;
  float ct_to_samp, offset;

  SYSTEMConfig(SYS_FREQ, SYS_CFG_ALL);

  numsamps = ISRFREQ/BASEFREQ;

  // bound checking
  if (numsamps > MAXSAMPS) numsamps = MAXSAMPS;

  // Should also perform bound checking on other variables
  // as mentioned in the great comment block at the top of the
  // program. -NR
  /* For example,
	if(BASEASMP is too high) {
		do something about it;
	}

  */
  
  // calculate useful variables
  ct_to_samp = (float)2*PI/numsamps;
  offset = (float) (PHASE/360)*numsamps;

  // initialize lookup table of duty cycle values
  for(ii=0;ii<numsamps;ii++) {
	dutyvec[ii] = (int)(500 + (500/1.65)*(BASEAMP*sin(ii*ct_to_samp)
		+HARMAMP*sin(HARMONIC*ii*ct_to_samp+offset)));

	// check bounds
	if (dutyvec[ii] > 999) dutyvec[ii] = 999;
	if (dutyvec[ii] < 0) dutyvec[ii] = 0;
  }

  // open 10 kHz timer
  OpenTimer4(T4_ON | T4_SOURCE_INT | T4_PS_1_8, 
	  SYS_FREQ/ISRFREQ/PRE_SCALER - 1);

  // set up the timer interrupt with a priority of 7
  ConfigIntTimer4(T4_INT_ON | T4_INT_PRIOR_7);

  // enable multi-vector interrupts
  INTEnableSystemMultiVectoredInt();  
  
  OpenTimer2(T2_ON | T2_PS_1_4, 1000); // 20kHz PWM: 80000000Hz / 4ps / 1000 = 20kHz
  // set up PWM output on Pin D0, 0 duty cycle
  OpenOC1(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0);

  return 0;
} // end main

/* ISR for Timer 4*/
void __ISR(_TIMER_4_VECTOR, ipl7) Timer4Handler(void){
    
  SetDCOC1PWM(dutyvec[count]); // set duty cycle

  // increment counter, reset at numsamps
  if(count < numsamps) {
	  count = count+1;
  }
  else {
	  count = 0;
  }

  // we can also write the above if/else as:
  // count = count < numsamps ? count+1 : 0;

  // clear the interrupt flag
  mT4ClearIntFlag();
}
